#ifndef _MLS_H
#define _MLS_H

#define XATTR_NAME_MLS "security.rba_mls"

#define MLS_FLAG_UNDEF 0x00000000 /* mac_mls invalid */
#define MLS_FLAG_SINGLE 0x00000001 /* mm_single initialized */
#define MLS_FLAG_RANGE 0x00000002 /* mm_range* initialized */
#define MLS_FLAG_INHERIT 0x00000004 /* mm_inherit initialized */
#define MLS_FLAG_KDE 0x00000008 /* mm_inherit initialized */

#define MLS_FLAGS_BOTH (MLS_FLAG_SINGLE | MLS_FLAG_RANGE)

#define MLS_TYPE_UNDEF 0 /* Undefined */
#define MLS_TYPE_LEVEL 1 /* Hierarchal level with mm_level */
#define MLS_TYPE_LOW 2 /* Dominated by any MAC_MLS_TYPE_LABEL */
#define MLS_TYPE_HIGH 3 /* Dominates any MAC_MLS_TYPE_LABEL */
#define MLS_TYPE_EQUAL 4 /* Equivilent to any MAC_MLS_TYPE_LABEL */

#define MLS_MAX_COMPARTMENTS 256

#define MAC_MAX_LEN 2048

struct mls_element {
	u_short type;
	u_short level;
	u_char compartments[MLS_MAX_COMPARTMENTS >> 3];
};

/*
 * MLS labels consist of two components: a single label, and a label
 * range.  Depending on the context, one or both may be used; the mb_flags
 * field permits the provider to indicate what fields are intended for
 * use.
 *
 * change the effective to single
 */
struct mls_label {
	int flags;
	struct mls_element single;
	struct mls_element kdesingle; // use for KDE
	struct mls_element kdetemp; // use for KDE
	struct mls_element rangelow, rangehigh;
	int initialized;
};

/*
 * MLS compartments bit test/set macros.
 * The range is 1 to MAC_MLS_MAX_COMPARTMENTS.
 */
#define MLS_BIT_TEST(b, w) ((w)[(((b) - 1) >> 3)] & (1 << (((b) - 1) & 7)))
#define MLS_BIT_SET(b, w) ((w)[(((b) - 1) >> 3)] |= (1 << (((b) - 1) & 7)))
#define MLS_BIT_SET_EMPTY(set) mls_bit_set_empty(set)

struct mls_label *mls_internalize_label(char *element_data);
int mls_externalize_label(struct mls_label *label, char *mlsstring);
int mls_free(void *obj_d);
struct mls_label *mls_init(void);
int set_fmls(const char *fn, char *capt, int *rv);
int get_fmls(const char *path_p, char *mlsstring);
struct mls_label get_pmls(const char *path_p);

ssize_t get_file_mls_attr(const char *path, void **value);
int set_file_mls_attr(const char *path, void *value);
ssize_t get_proc_mls_attr(pid_t pid, void **value);

#endif
